import os.path
import os
import time
import sys
from mx.DateTime import DateTime,DateTimeDelta

class Context:
    def __init__(self, root, logfile):
        self.log = logfile
        self.root = root
        self.src = self.joinAndCheck(self.root,"src")
        self.PythonTools = self.joinAndCheck(self.src,"PythonTools")
        self.pageSources = self.joinAndCheck(self.src,"pageSources")
        self.figureSources = self.joinAndCheck(self.src,"figureSources")
        self.noprefix = self.joinAndCheck(self.figureSources,"noprefix")
        self.unpublished = self.joinAndCheck(self.figureSources,"unpublished")
        self.documentSources = self.joinAndCheck(self.src,"documentSources")
        self.weblog = self.joinAndCheck(self.src,"weblog")
        self.zippedLogfiles = self.joinAndCheck(self.src,"zippedLogfiles")
        self.www = self.joinAndCheck(self.root,"www")
        self.zip = self.joinAndCheck(self.www,"sources")
        self.documents = self.www
        self.wwwWeblog = self.joinAndCheck(self.www,"weblog")
        self.temp = self.joinAndCheck(self.root,"temp")
        self.cache = self.joinAndCheck(self.root,"cache")
        self.figures = self.joinAndCheck(self.cache,"figures")
        self.figCrossRef = self.joinAndCheck(self.cache,"figCrossRef")
        self.logAnalyse = self.joinAndCheck(self.cache,"logAnalyse")
        self.logRaw = self.joinAndCheck(self.cache,"lograw")
        self.texLog = self.joinAndCheck(self.cache,"texLog")
        self.local = self.joinAndCheck(self.root,"local")
        self.localWeblog = self.joinAndCheck(self.local,"weblog")
        self.wwwfigures = self.joinAndCheck(self.www,"figures")
##        os.chdir(rootdir)
##        files = os.listdir(self.rootdir)
##        for f in files:
##            if os.path.isdir(f):
##                self.__dict__[f] = self.checkPath(os.path.join(rootdir,f))
        self.visio = 0
        self.sequence = 0
        
    def joinAndCheck( self, root, subpath, message=" path does not exist"):
        path = os.path.join(root,subpath)
        if not os.path.exists(path):
            self.log.fatal(path + message)
        return path
    
    def checkPath( self, path, message=" path does not exist"):
        if not os.path.exists(path):
            self.log.fatal(path + message)
        return path
    
    def checkFile( self, file, message=" file does not exist"):
        if not os.path.isfile(file):
            self.log.info(file + message)
            return ""
        return file
    
    def getDef( self, defFile, attrName ):
        if self.checkFile( defFile )=="":
            return ""
        file = open(defFile,"r")
        lines = file.readlines()
        file.close()
        for line in lines:
            elements = line.split("=")
            if elements[0].strip()==attrName:
                return elements[1].strip()
        return ""

    def fileToTitle(self, fileName,ext):
        if ext==".pdf":
            postFix = ["Book","Paper","allSlides","Slides"]
            fileRoot = fileName
            type = "document"
            for p in postFix:
                if fileName[-len(p):]==p:
                    fileRoot = fileName[:-len(p)]
                    type = p
                    break
            docdef = os.path.join(self.documentSources,fileRoot,fileRoot+".DocDef.txt")
            if self.checkFile( docdef )=="":
                docdef = os.path.join(self.documentSources,"nonGeneratedDocuments",
                                      fileRoot+".DocDef.txt")
                if self.checkFile( docdef )=="":
                    docdef = os.path.join(self.documentSources,"EasyVision",
                                          fileName+".DocDef.txt")
                    type = "old EV document"
                    if self.checkFile(docdef)=="":
                        return ("","")
            return (type, self.getDef(docdef,"title"))
        if ext==".html":
            pagedef = os.path.join(self.pageSources,fileName+".PageDef.txt")
            return ("page",self.getDef(pagedef,"Title"))
        if ext==".ppt":
            docdef = os.path.join(self.documentSources,"nonGeneratedDocuments",
                                  fileName+".DocDef.txt")
            return ("presentation",self.getDef(docdef,"title"))
        if ext==".zip":
            docdef = os.path.join(self.documentSources,fileName,fileName+".DocDef.txt")
            if self.checkFile(docdef)<>"":
                return ("document",self.getDef(docdef,"title"))
        return ("","")
    
    def close(self):
        self.log.close()
        
    def makeGLAfilename( self,date,level, ftype):
        if ftype =="raw":
            return os.path.join(self.logRaw,makeDateId(date,level)+"."+ftype)
        else:
            return os.path.join(self.logAnalyse,makeDateId(date,level)+"."+ftype)
    
    def makeHtmlFilename(self,date,level,ftype):
        return "index"+makeDateId(date,level)+"."+ftype+".html"

    def makeHtmlPathname(self,date,level,ftype):
        return os.path.join(self.localWeblog,
                            self.makeHtmlFilename(date,level,ftype))

    def makeWwwPathname(self,date,level,ftype):
        return os.path.join(self.wwwWeblog,
                            self.makeHtmlFilename(date,level,ftype))
    
    def getDateRange(self):
        files = os.listdir(self.logRaw)
        files.sort()
        firstfile = files[0]
        lastfile = files[-1]
        beginDate = DateTime(int(firstfile[0:4]),
                                        int(firstfile[4:6]),int(firstfile[6:8]))
        beginDate -= DateTimeDelta(6)
        endDate = DateTime(int(lastfile[0:4]),
                                        int(lastfile[4:6]),int(lastfile[6:8]))
        return beginDate,endDate
    
dateFormatString = ["all","%Y","%Y%m","%Y%m%d"]
def makeDateId(date,level):
    return date.strftime(dateFormatString[level])

class LogFile:
    def __init__(self, basename,toFile=1,toWindow=0,showInfo=0):
        self.toFile = toFile
        self.toWindow = toWindow
        self.showInfo = showInfo
        if self.toFile:
            self.logfile = open(basename+
                                time.strftime(".%Y%m%d.%H%M",time.localtime(time.time()))+".log","w")
        else:
            self.logfile = 0

    def fatal( self, message ):        
        self.log(message)
        self.logfile.close()
        sys.exit("fatal error; "+message)
            
    def log( self, message ):
        if self.toFile:
            self.logfile.write( message +"\n")
        if self.toWindow:
            print message

    def info( self, message ):
        if self.showInfo:
            self.log(message)

    def close( self ):
        if self.logfile<>0:
            self.logfile.close()

def main():               
    rootdir = "f:\\gaudi"
    logFile = LogFile("f:\\gaudi\\temp\\testContext",showInfo=0)
    startTime = time.time()
    context = Context(rootdir,logFile)
    for attr in context.__dict__.keys():
        print "context."+attr," = ",context.__dict__[attr]
    print context.getDateRange()
    endTime = time.time()
    logFile.log("finished at: "+ time.ctime(endTime))
    logFile.log("total elapsed time:%7.1f seconds"%(endTime - startTime))
    logFile.close()

if __name__ == '__main__':
    main()
    
