from __future__ import generators
from mx.DateTime import DateTime,DateTimeDelta
from mx.Tools import sizeof
import string
import os
import shutil
import os.path
from Timer import Timer
import ipreversal
import pprint
from HTMLgen import *

from GLAcontext import *
from GLAtypes import *
from GLAmakeStats import *
from GLAoutput import *
from GLApickle import *
from GLAmakeEntries import *
from GLAdate import *
from GLAgraph import *
from GLAtransformESIfiles import *

class AnalyzerData:
    def __init__(self,context):
        self.ipDB = ipreversal.IPdb(os.path.join(context.PythonTools,"ipreverseNew.txt"))
        self.searchEngines=[
            "fastsearch.net",
            "googlebot.com",
            "nj.nec.com",
            "wib.pa-x.dec.com",
            "directhit.com",
            "inktomisearch.com",
            "club-internet.fr",
            "crawler918.com",
            "teoma.com",
            "public.alexa.com",
#            "131.107.163.50",
            "searchengine.microsoft.com",
            "netcache7.is.co.za",
            "turnitin.com",
            "trinity.it",
            "selenia3.united.net.ua",
            "61.78.59-63.kornet.kr",
            "pr7-ts.telepac.pt",
            "datela-1-6-34.dialup.vol.cz",
            "pool.mediaways.net",
            "looksmart.com",
            ".sv.av.com",
            "crawler.kototoi.org",
            "SHENZHEN-HTP-CO.cn",
            "kc13394.eesi.tue.nl",
            "cma.com",
            "test-ipcms.u-strasbg.fr",
            "m74y12.is-energy.de",
            "debis.de",
            "search.msn.com",
            "4.piok2.xdsl.nauticom.net",
            "c-24-0-31-118.client.comcast.net",
            "cp538812-a.venra1.lb.home.nl",
            "cache1.asianet.co.th",
            "207.46.microsoft.com",
            ".dir.com",
            "search.aol.com",
            "dyn-144041.nbw.tue.nl",
            "195-240-183-207-mx.xdsl.tiscali.nl",
            "203.145.160-191.bhartibroadband.com",
            "urz.TU-Dresden.de",
            "cp454373-a.dbsch1.nb.home.nl",
            "c51449555.cable.wanadoo.nl",
            "chinanet.bot.sh.cn",
            "XA-YC-MIDD-SCH.shaanxi.cn",
            "sanshui.jlb.ltd.cn",
            "adsl-211-78-141-140.bc.sparqnet.net",
            "jci.com",
            "p3ee3cb91.dip.t-dialin.net",
            "REGUSUS23.net",
            "61.11.59.dishnet.in",
            "fia136-115.dsl.hccnet.nl",
            "webcrawlers.inktomi.com",
            "msnbot.msn.com",
            "isu.net.sa",
            "egspd42259.ask.com",
            "waseda.jp",
            "66.150.40.InternetSeer.net",
            "google.66.249.com",
            "scaea-http-2.sun.com",
            "ask.com",
            "dyn38-096.medewerker.mu.hro.nl",
            ".exabot.com",
            "c-67-173-188-237.hsd1.il.comcast.net",
            "SevenTwentyfour.unknown",
            "61.77.kornet.kr",
            "concerto.com",
            "h-67-101-89-232.phndaz91.dynamic.covad.net",
            "webcache-kulnet-2.kuleuven.ac.be",
            "addr-85-23-12-165.suomi.net",
            ]
        self.nameChanges={
            ".pdf":{
                "SystemArchitecture":"SystemArchitectureBook",
                "SupportingProcesses":"SupportingProcessesBook",
                "ScenarioHowToPaper":"StoryHowToPaper",
                "ScenarioHowToSlides":"StoryHowToSlides",
                "PFBusinessAnalysisAndDefinitionSlides":"PFAnalysisAndDefinitionSlides",
                "PFBusinessAnalysisAndDefinitionPaper":"PFAnalysisAndDefinitionPaper",
                },
            ".zip":{
                "ScenarioHowTo":"StoryHowTo",
                "PFBusinessAnalysisAndDefinition":"PFAnalysisAndDefinition",
                },
            }
        self.types = ["pdf","html","ppt","zip","figs","stat",""]
        self.typesSets = []
        for aType in self.types:
            for aSet in ["philips","ROW"]:
                self.typesSets.append(aType+"."+aSet)
        self.wwwTypesSets = []
        for aType in self.types:
            self.wwwTypesSets.append(aType+".ROW")        
        self.infoTitle = {"IPhist":"IP statistics",
                          "domain":"Statistics of top domains",
                          "IPfreq":"Histogram of hits/ip address",
                          "over":"overview",
                          "log":"Log entries",
                          "overall":"overview all days",
                          "log4xx":"log entries with error status 4xx",
                          "4xx":"accesses with error status 4xx",
                          "search":"search engine access count",
                          "numip":"numerical IP addresses"}
        self.statFiles = ["counters","IPhist","log4xx","4xx","search"]
        self.aggregatedStatFiles = ["stat","IPhist"]
        self.counters = ["size","4xx","304","log4xx","search"]
        self.IPinfo = ["domain","freq","hist"]
        for aTypeSet in self.typesSets:
            self.infoTitle[aTypeSet] = aTypeSet+ " files"
            self.statFiles.append(aTypeSet)
            self.counters.append(aTypeSet)
        print self.counters
        self.htmlDailyFiles = ["log4xx","log"]
        self.htmlNonDailyFiles = [
            "over",
            "4xx",
            "search",
            "domain",
            "IPfreq",
            "IPhist"] +self.typesSets
        self.wwwIdenticalFiles = [
            "4xx",
            "search",
            "domain"] +self.typesSets
        
    def translateFilename(self,filename,ext):
        if ext in self.nameChanges:
            if filename in self.nameChanges[ext]:
                return self.nameChanges[ext][filename],ext
        return filename,ext

    def generatePageTitle(self,date, level, info):
        return Header(2,self.infoTitle[info]+" "+makeDateLabel(date,level),"\n")

def deleteForRenewedSearch(startDate, endDate, context):
    for date in dayGenerator(startDate,endDate):
        if os.path.isfile(context.makeGLAfilename(date,3,"hits")):
            os.remove(context.makeGLAfilename(date,3,"hits"))
        if os.path.isfile(context.makeGLAfilename(date,3,"IPhist")):
            os.remove(context.makeGLAfilename(date,3,"IPhist"))
def deleteForRenewedIPreverse(startDate,endDate,context):
    for date in dayGenerator(startDate,endDate):
        if os.path.isfile(context.makeGLAfilename(date,3,"IPhist")):
            os.remove(context.makeGLAfilename(date,3,"IPhist"))

def deleteChangedData(datesChanged,analyzerData,context):
    for date in datesChanged:
        for ftype in analyzerData.statFiles:
            if os.path.isfile(context.makeGLAfilename(date,3,ftype)):
                os.remove(context.makeGLAfilename(date,3,ftype))
        for ftype in analyzerData.htmlDailyFiles:
            if os.path.isfile(context.makeHtmlFilename(date,3,ftype)):
                os.remove(context.makeHtmlFilename(date,3,ftype))

def processDays(startDate,endDate,htmlStartDate,datesChanged,showLevel,analyzerData,context):
    monthChanges = {}
    for date in dayGenerator(startDate,endDate):
        if showLevel>0:
            if date.day==1:
                print
                print date.year,"-",date.month,"-",date.day,
            else:
                print date.day,
        changed = makeStats(date,context,analyzerData)
        makeIPhist(date,context,analyzerData,changed)
        if date>htmlStartDate:
            if (not os.path.isfile(context.makeHtmlPathname(date,3,"log"))) or date in datesChanged:
                makePage(date,3,"log",context,analyzerData)
            if (not os.path.isfile(context.makeHtmlPathname(date,3,"log4xx"))) or "log4xx" in changed:
                makePage(date,3,"log4xx",context,analyzerData)
##        monthId = (date.year,date.month)
##        if monthId in monthChanges:
##            for change in changed:
##                if not change in monthChanges[monthId]:
##                    monthChanges[monthId].append(change)
##        else:
##            monthChanges[monthId]=changed
    return monthChanges

def processMonths(startDate,endDate,monthChanges,analyzerData,context):
    startMonth = DateTime(startDate.year,startDate.month)
    for date in monthGenerator(startMonth,endDate):
##        monthId = (date.year,date.month)
        for ftype in analyzerData.aggregatedStatFiles:
##            if not os.path.isfile(context.makeGLAfilename(date,2,ftype)):
##                if monthId in monthChanges:
##                    if not ftype in monthChanges[monthId]:
##                        monthChanges[monthId].append(ftype)
##                else:
##                    monthChanges[monthId] = [ftype]
##        if monthId in monthChanges:
##            for ftype in monthChanges[monthId]:
            mergeStats(date,3,ftype,context)
def outputMonths(startDate,endDate,analyzerData,context):
    for date in monthGenerator(DateTime(startDate.year,startDate.month),endDate):
        for ftype in analyzerData.htmlNonDailyFiles:
            makePage(date,2,ftype,context,analyzerData)
        for ftype in analyzerData.wwwIdenticalFiles:
            shutil.copyfile(context.makeHtmlPathname(date,2,ftype),
                            context.makeWwwPathname(date,2,ftype))
        makePage(date,2,"over",context,analyzerData,1)
    
def main(renewSearch=0,renewIP=0,showLevel=0):
    t = Timer()
    drive = os.path.splitdrive(os.getcwd())[0]
    root = os.path.join(drive,"/gaudi")
    context = Context(root,LogFile(os.path.join(root,"temp\\GLA"),
                                   toWindow=1,showInfo=0))
    analyzerData = AnalyzerData(context)
    IPdict = {}
##    transformMonthlyESIzipfile(context, "200506.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200507.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200508.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200509.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200510.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200511.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200512.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200601.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200602.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200603.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200604.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200605.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200606.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200607.zip",IPdict)
##    transformMonthlyESIzipfile(context, "200608.zip",IPdict)
    transformMonthlyESIzipfile(context, "200609.zip",IPdict)
    transformMonthlyESIzipfile(context, "200610.zip",IPdict)
    startDate = DateTime(2006,9,1)
    endDate = DateTime(2006,10,23)
    datesChanged=[]
##    startDate, endDate, datesChanged = makeEntryFiles(context)
    print t.stamp("makeEntryFiles finished")
    if renewSearch:
        deleteForRenewedSearch(startDate, endDate, context)
        print t.stamp("hits and IPhist files removed from cache")
    if renewIP:
        deleteForRenewedIPreverse(startDate,endDate,context)
        print t.stamp("IPhist files removed from cache")
    deleteChangedData(datesChanged,analyzerData,context)
    monthChanges = processDays(startDate,endDate,endDate - DateTimeDelta(60),
                               datesChanged,showLevel,analyzerData,context)
    print t.stamp("days statistics and HTML generated")
    monthChanges={}
    processMonths(startDate,endDate,monthChanges,analyzerData,context)
    print t.stamp("months statistics made")
    outputMonths(startDate,endDate,analyzerData,context)
    print t.stamp("months generated")
    for date in [DateTime(2002),DateTime(2003),DateTime(2004),DateTime(2005),DateTime(2006)]:
        for ftype in analyzerData.aggregatedStatFiles:
            mergeStats(date,2,ftype,context)
    print t.stamp("years statistics made")
    for date in [DateTime(2002),DateTime(2003),DateTime(2004),DateTime(2005),DateTime(2006)]:
        for ftype in analyzerData.htmlNonDailyFiles:
            if os.path.isfile(context.makeHtmlPathname(date,1,ftype)):
                os.remove(context.makeHtmlPathname(date,1,ftype))
            print date, ftype
            makePage(date,1,ftype,context,analyzerData)
        for ftype in analyzerData.wwwIdenticalFiles:
            shutil.copyfile(context.makeHtmlPathname(date,1,ftype),
                    context.makeWwwPathname(date,1,ftype))
        makePage(date,1,"over",context,analyzerData,1)
    print t.stamp("years generated")
    for ftype in analyzerData.aggregatedStatFiles:
        mergeStats(DateTime(2002),1,ftype,context)
    for ftype in analyzerData.htmlNonDailyFiles:
        if os.path.isfile(context.makeHtmlPathname(DateTime(2002),0,ftype)):
            os.remove(context.makeHtmlPathname(DateTime(2002),0,ftype))
        makePage(DateTime(2002),0,ftype,context,analyzerData)
    for ftype in analyzerData.wwwIdenticalFiles:
        shutil.copyfile(context.makeHtmlPathname(DateTime(2002),0,ftype),
                        context.makeWwwPathname(DateTime(2002),0,ftype))
    makePage(DateTime(2002),0,"over",context,analyzerData,www=1)
    makePage(DateTime(2002),0,"numip",context,analyzerData,www=0)
    makeOverviewGraph(DateTime(2002,5),endDate,context,analyzerData)
    print t.stamp("finished")
    context.close()
    
if __name__ == '__main__':
    main( renewSearch=1, renewIP=1,showLevel=2 )
