import os
import os.path
import copy
from GLApickle import *
from GLAdate import *
from GLAtypes import *

def alldigits(ip):
    ipelements = ip.split(".")
    alldigit = 1
    for el in ipelements:
        alldigit = alldigit and el.isdigit()
    return alldigit

def translateIP(ip,ipDB):    
    if alldigits(ip):
        return ipDB.get(ip)
    else:
        return ip

def filterTypes(entries,fileTypes):
    filtered = []
    for e in entries:
        if e.ext in fileTypes:
            filtered.append(e)
    return filtered

def splitPhilips(entries):
    philips = []
    nonPhilips = []
    for e in entries:
        if e.domain[-len("philips.com"):]=="philips.com":
            philips.append(e)
        else:
            nonPhilips.append(e)
    return philips, nonPhilips

def splitSearch(entries,searchEngines):
    setsNonSearch = []
    searchHist = Hist()
    for e in entries:
        for s in searchEngines:
            if e.domain[-len(s):]==s:
                searchHist.add(s)
                break
        else:
            setsNonSearch.append(e)
    return setsNonSearch, searchHist

def countGoogle(entries):
    count = 0
    for entry in entries:
        if entry.file=="GaudiGoogleLogo" and entry.ext=="gif":
            count += 1
    return count

def splitStatus4xx(entries):
    status4xxHist = Hist()
    status4xx = []
    statusOK = []
    for e in entries:
        for s in e.status:
            if s[0]=="4":
                status4xxHist.add((e.command,e.dir,e.file,e.ext,s))
                status4xx.append(e)
            else:
                statusOK.append(e)
                break
    return statusOK,status4xx,status4xxHist

def splitExts(entries,analyzerData):
    exts={}
    for ext in analyzerData.types:
        exts[ext]=Hist()
    for e in entries:
        fileName,ext = analyzerData.translateFilename(e.file,e.ext)
        if ext in analyzerData.types:
            if "/figures" in e.dir:
                exts["figs"].add((fileName,ext))
            elif "/weblog" in e.dir:
                exts["stat"].add((fileName,ext))
            else:
                exts[ext].add((fileName,ext))
    return exts

def sortTime(a,b):
    return a.datetime>b.datetime

def collapseEntries(entries):
    collapsed = {}
    for d in entries[1:]:
        id = (d.ipadres,d.datetime.year,d.datetime.month,d.datetime.day,d.file,d.ext)
        if id in collapsed:
            collapsed[id].length+=d.length
            collapsed[id].status+=d.status
        else:
            collapsed[id]=copy.deepcopy(d)
    collapsedList = collapsed.values()
    collapsedList.sort(sortTime)
    return collapsedList

def makeStats( day, context, analyzerData ):
    if os.path.isfile(context.makeGLAfilename(day,3,"stat")):
        return []
    changed = []
    filenameRaw = context.makeGLAfilename(day,3,"raw")
    try:
        inf = open(filenameRaw,"rb")
        setRaw = cPickle.load(inf)
        inf.close()
    except EOFError:
        inf.close()
        context.log.log("file "+filenameRaw+" unpickling gives EOF message")
        return []
    for e in setRaw:
        e.domain = translateIP(e.domain,analyzerData.ipDB)
    stats={}
    setCollapsed = collapseEntries(setRaw)
    setStatusOK,setStatus4xx,status4xxHist = splitStatus4xx(setCollapsed)
    stats["4xx"]=status4xxHist
    if not os.path.isfile(context.makeGLAfilename(day,3,"log4xx")):
        pickleToFile(context.makeGLAfilename(day,3,"log4xx"),setStatus4xx)
        changed.append("log4xx")
    setNonSearch, searchHist = splitSearch(setStatusOK,analyzerData.searchEngines)
    stats["search"]=searchHist
    setTypes=filterTypes(setNonSearch,analyzerData.types)
    set = {}
    set["philips"],set["ROW"]=splitPhilips(setTypes)
    IPhistFileName = context.makeGLAfilename(day,3,"IPhist")
    if not os.path.isfile(IPhistFileName):
        theIPhist = IPhist(set["ROW"],context,analyzerData.ipDB)
        pickleToFile(IPhistFileName,theIPhist)
        changed.append("IPhist")
    exts={}
    for aSet in ["philips","ROW"]:
        exts[aSet]=splitExts(set[aSet],analyzerData)
        for ext in exts[aSet].keys():
            if ext in analyzerData.types:
                stats[ext+"."+aSet]=exts[aSet][ext]
    counters = Counters(len(setRaw),analyzerData)
    for e in setRaw:
        counters.add("size",e.length)
        for s in e.status:
            if s[0]=="4":
                counters.add("4xx",1)
            if s=="304":
                counters.add("304",1)
    counters.add("collapsed",len(setCollapsed))
    counters.add("log4xx",len(setStatus4xx))
    counters.add("search",searchHist.total)
    counters.add("google",countGoogle(setNonSearch))
    counters.add("days",1)
    for aSet in ["philips","ROW"]:
        counters.add(aSet,len(set[aSet]))
        for ext in analyzerData.types:
            counters.add(ext+"."+aSet, exts[aSet][ext].total)
##                for f in exts[aSet][ext].hist.values():
##                    counters.add(ext+"."+aSet,f)
    changed.append("stat")
    pickleToFile(countersFileName,(stats,counters))
    return changed

def mergeStats(startDate,level,ftype,context):
    if ftype=="log4xx":
        print "panic log4xx mergeStats"
        return
    sourceGenerator = dateGenerator(startDate,level)
    date = sourceGenerator.next()
    while not os.path.isfile(context.makeGLAfilename(date,level,ftype)):
        date = sourceGenerator.next()
    content = unPickleFile(context.makeGLAfilename(date,level,ftype),context)
    for date in sourceGenerator:
        file = context.makeGLAfilename(date,level,ftype)
        if os.path.isfile(file):
            if ftype=="stat":
                stat1,counters1=content
                stat2,counters2 = unPickleFile(file),context)
                for statType in stat1.keys():
                    stat1[statType].merge(stat2[statType])
                counters1.merge(counters2)
                content = stat1,counters1
            else:
                stat.merge(unPickleFile(file),context)
    pickleToFile(context.makeGLAfilename(startDate,level-1,ftype), stat)

