from __future__ import generators
from mx.DateTime import DateTime,DateTimeDelta
from mx.Tools import sizeof
import string
import os
import os.path
from GLAcontext import *
from GLAdate import *
from GLAtypes import *
from HTMLgen import *
from GLApickle import *

dateLabelFormatString = ["all","%Y","%Y month: %m","%Y-%m-%d"]
def makeDateLabel(date,level):
    return date.strftime(dateLabelFormatString[level])

def generateNavigation(info, date, level,context,analyzerData):
    # home in case of nlww to be implemented
    home = "http://www.gaudisite.nl/"
    nav = TR(TH(Href(home,"home "),bgcolor="beige"))
    if level>0 and not info in ["log","log4xx"]:
        nav.append(TH(Href(context.makeHtmlFilename(date,level-1,info),
                           analyzerData.infoTitle[info]+" "+makeDateId(date,level-1)),
                      bgcolor="beige"))
    if info<>"over" and level<3:
        nav.append(TH(Href(context.makeHtmlFilename(date,level,"over"),
                           "overview "+makeDateId(date,level)),bgcolor="beige"))
    if info in ["log","log4xx"] and level==3:
        nav.append(TH(Href(context.makeHtmlFilename(date,level-1,"over"),
                           "overview "+makeDateId(date,level-1)),bgcolor="beige"))
    return TableLite(nav,border=1,bordercolor="brown",cellpadding=2,cellspacing=10)
    
def makePage( date,level,info,context,analyzerData,www=0):
    doc = SimpleDocument()
    doc.append(analyzerData.generatePageTitle(date,level,info))
    doc.append(generateNavigation(info,date,level,context,analyzerData))
    doc.append(BR(2),"\n")
    if info=="log":
        doc.append(makeLogEntries(date,context))
    elif info == "log4xx":
        doc.append(make4xxEntries(date,context))
    elif info == "domain":
        doc.append(makeDomainTable(date, level, context))
    elif info == "IPfreq":
        doc.append(makeIPfreqTable(date, level, context))
    elif info == "IPhist":
        doc.append(makeIPhistTable(date, level, context, analyzerData))
    elif info == "numip":
        doc.append(makeNumIPtable(date,level,context,analyzerData))
    elif info == "search":
        doc.append(makeSearchTable(date, level, context))
    elif info == "4xx":
        doc.append(make4xxTable(date, level, context))
    elif info in analyzerData.typesSets:
        doc.append(makeDocTable(date, level, info, context))
    elif info == "over":
        if level==3:
            print "level 3 not allowed"
        if www:
            doc.append(makeWwwOverviewTable(date, level, context, analyzerData))
        else:
            doc.append(makeOverviewTable(date, level, context, analyzerData))
    else:
        print "? date, level, info:",date,level,info
    if www:
        doc.write(context.makeWwwPathname(date,level,info))
    else:
        doc.write(context.makeHtmlPathname(date,level,info))

def makeDocTable( date, level, info, context ):
    stat,counters = unPickleFile(context.makeGLAfilename(date, level, "stat"),context)
    stats = stat[info]
    tab = TableLite(TR(TH("count"),TH("title"),TH("file")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                       "<COLGROUP><COL ALIGN=LEFT></COLGROUP>"
                       "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.getHist(1):
        type,title = context.fileToTitle(key[0],"."+key[1])
        fileName = key[0]+"."+key[1]
        if title=="":
            title = "title unknown"
        if os.path.isfile(os.path.join(context.www,fileName)):
            label = Href("../"+fileName,title)
        else:
            label = title
        tab.append(TR(TD(val),TD(Text(type+":\t"),label), TD(fileName)),"\n")
    return tab        

def makeSearchTable( date, level, context ):
    stat,counters = unPickleFile(context.makeGLAfilename(date, level, "stat"),context)
    stats = stat["search"] 
    tab = TableLite(TR(TH("count"),TH("bot a.o. search-engine")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                   "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.getHist(1):
        tab.append(TR(TD(val),TD(key)),"\n")
    return tab        

def make4xxTable( date, level, context ):
    stat,counters = unPickleFile(context.makeGLAfilename(date, level, "stat"),context)
    stats = stat["4xx"]
    tab = TableLite(TR(TH("count"),TH("4xx error")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                   "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.getHist(1):
        tab.append(TR(TD(val),TD(key)),"\n")
    return tab        

def makeDomainTable( date, level, context ):
    stats = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    tab = TableLite(TR(TH("count"),TH("accumulated",BR(),"hits"),
                       TH("average",BR(),"hits/IP"),TH("topdomain")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                       "<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                       "<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                       "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.top.getHist(1):
        tab.append(TR(TD(val[0]),TD(val[1]),
                      TD("%7.2f"%(float(val[1])/float(val[0]))),TD(key)),"\n")
    return tab        

def makeIPfreqTable(date, level,context):
    stats = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    tab = TableLite(TR(TH("count"),TH("freq")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                   "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for count in range(len(stats.freqHist)):
        tab.append(TR(TD(count),TD(stats.freqHist[count])),"\n")
    return tab        

def makeIPhistTable(date, level,context, analyzerData):
    stats = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    tab = TableLite(TR(TH("count"),TH("domain")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                   "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.hist.getHist(1):
        if val>10:
            tab.append(TR(TD(val),TD(key), TD(analyzerData.ipDB.getDescription(key))),"\n")
    return tab        

def makeNumIPtable(date, level,context, analyzerData):
    stats = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    tab = TableLite(TR(TH("count"),TH("ipaddress")),"\n")
    colgroup = RawText("<COLGROUP ALIGN=RIGHT></COLGROUP>"+
                   "<COLGROUP><COL ALIGN=LEFT></COLGROUP>")
    tab.append(colgroup)
    for val,key in stats.hist.getHist(1):
        if alldigits(key):
            tab.append(TR(TD(val),TD(key)),"\n")
    return tab        

def makeLogEntries( date, context):
    if os.path.isfile(context.makeGLAfilename(date,3,"raw")):
        entries = unPickleFile(context.makeGLAfilename(date,3,"raw"),context)
        tab = TableLite()
        for d in entries:
            row = TR(TD(d.ipadres+" "+d.domain))
            row.append(TD(d.datetime.strftime("%H:%M:%S")))
    ##        row.append(TD(d.protocol))
            row.append(TD(d.file+"."+d.ext))
            el=TD(d.status[0])
            for s in d.status[1:]:
                el.append(","+s)
            row.append(el)
            row.append(TD(d.length),"\n")
            tab.append(row)
        return tab
    else:
        return TableLite()

def make4xxEntries( date, context):
    entries = unPickleFile(context.makeGLAfilename(date,3,"log4xx"),context)
    tab = TableLite()
    for d in entries:
        row = TR(TD(d.ipadres))
        row.append(TD(d.datetime.strftime("%H:%M:%S")))
        row.append(TD(d.command))
##        row.append(TD(d.protocol))
        row.append(TD(d.dir+"/"+d.file+"."+d.ext))
        el=TD(d.status[0])
        for s in d.status[1:]:
            el.append(","+s)
        row.append(el)
        row.append(TD(d.length),"\n")
        tab.append(row)
    return tab

def makeOverviewTable(date,level,context,analyzerData):
    tab = TableLite()
    generateHeader(tab,analyzerData.types)
    for aDate in dateGenerator(date,level+1):
        if os.path.isfile(context.makeGLAfilename(aDate, level+1, "IPhist")):
            IPhist = unPickleFile(context.makeGLAfilename(aDate, level+1, "IPhist"),context)
            stats, counters = unPickleFile(context.makeGLAfilename(aDate, level+1, "stat"),context)
            if level==2:
                tab.append(generateDailyRow(aDate,counters,IPhist,context,analyzerData))
            else:
                tab.append(generateRow(aDate,level+1,counters,IPhist,context,analyzerData))
    IPhist = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    print date,level,context.makeGLAfilename(date,level,"stat")
    stats,counters = unPickleFile(context.makeGLAfilename(date, level, "stat"),context)
    tab.append(TR(TD(colspan=30,height=1,bgcolor="#000000")),"\n")
    tab.append(generateRow(date,level,counters,IPhist,context,analyzerData))
    return tab

def makeWwwOverviewTable(date,level,context,analyzerData):
    tab = TableLite()
    wwwGenerateHeader(tab,analyzerData.types)
    for aDate in dateGenerator(date,level+1):
        if os.path.isfile(context.makeGLAfilename(aDate, level+1, "IPhist")):
            IPhist = unPickleFile(context.makeGLAfilename(aDate, level+1, "IPhist"),context)
            stat,counters = unPickleFile(context.makeGLAfilename(aDate, level+1, "stat"),context)
            if level==2:
                tab.append(wwwGenerateDailyRow(aDate,counters,IPhist,context,analyzerData))
            else:
                tab.append(wwwGenerateRow(aDate,level+1,counters,IPhist,context,analyzerData))
    IPhist = unPickleFile(context.makeGLAfilename(date, level, "IPhist"),context)
    stat, counters = unPickleFile(context.makeGLAfilename(date, level, "stat"),context)
    tab.append(TR(TD(colspan=30,height=1,bgcolor="#000000")),"\n")
    tab.append(wwwGenerateRow(date,level,counters,IPhist,context,analyzerData))
    return tab        
    
def generateDailyRow(date,counters,IPhist,context,analyzerData):
    row = TR(TD(makeDateLabel(date,3),"\n"))
    row.append(TD(counters.get("size")/1024/1024),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,3,"log"),
                       counters.get("rawEntries"))),"\n")
    row.append(TD(counters.get("304")),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,3,"log4xx"),
                       counters.get("4xx"))),"\n")
    row.append(TD(counters.get("collapsed")),"\n")
    row.append(TD(counters.get("log4xx")),"\n")
    row.append(TD(counters.get("search")),"\n")
    row.append(TD(counters.get("google")),"\n")
    row.append(TD(IPhist.totalIP),"\n")
    row.append(TD(IPhist.domains),"\n")
    row.append(TD("%5.2f"%IPhist.IPaverage),"\n")
    for aSet in ["ROW","philips"]:
        row.append(TD(counters.get(aSet)),"\n")
        for aType in analyzerData.types:
            row.append(TD(counters.get(aType+"."+aSet)),"\n")
    return row

def generateRow(date,level,counters,IPhist,context,analyzerData):
    row = TR(TD(Href(context.makeHtmlFilename(date,level,"over"),
                     makeDateLabel(date,level)),"\n"))
    row.append(TD(counters.get("size")/1024/1024),"\n")
    row.append(TD(counters.get("rawEntries")),"\n")
    row.append(TD(counters.get("304")),"\n")
    row.append(TD(counters.get("4xx")),"\n")
    row.append(TD(counters.get("collapsed")),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"4xx"),
                       counters.get("log4xx"))),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"search"),
                       counters.get("search"))))
    row.append(TD(counters.get("google")),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"IPhist"),
                       IPhist.totalIP)),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"domain"),
                       IPhist.domains)),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"IPfreq"),
                       "%5.2f"%IPhist.IPaverage)),"\n")
    for aSet in ["ROW","philips"]:
        row.append(TD(counters.get(aSet)),"\n")
        for aType in analyzerData.types:
            row.append(TD(Href(context.makeHtmlFilename(date,level,aType+"."+aSet),
                               counters.get(aType+"."+aSet))),"\n")
    return row

def wwwGenerateRow(date,level,counters,IPhist,context,analyzerData):
    row = TR(TD(Href(context.makeHtmlFilename(date,level,"over"),makeDateLabel(date,level)),"\n"))
    row.append(TD(counters.get("size")/1024/1024),"\n")
    row.append(TD(counters.get("rawEntries")),"\n")
    row.append(TD(counters.get("304")),"\n")
    row.append(TD(counters.get("4xx")),"\n")
    row.append(TD(counters.get("collapsed")),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"4xx"),
                       counters.get("log4xx"))),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"search"),
                       counters.get("search"))))
    row.append(TD(counters.get("google")),"\n")
    row.append(TD(Href(context.makeHtmlFilename(date,level,"domain"),
                       IPhist.domains)),"\n")
    row.append(TD(counters.get("ROW")),"\n")
    for aType in analyzerData.types:
        row.append(TD(Href(context.makeHtmlFilename(date,level,aType+".ROW"),
                           counters.get(aType+".ROW"))),"\n")
    return row

def wwwGenerateDailyRow(date,counters,IPhist,context,analyzerData):
    row = TR(TD(makeDateLabel(date,3)),"\n")
    row.append(TD(counters.get("size")/1024/1024),"\n")
    row.append(TD(counters.get("rawEntries")),"\n")
    row.append(TD(counters.get("304")),"\n")
    row.append(TD(counters.get("4xx")),"\n")
    row.append(TD(counters.get("collapsed")),"\n")
    row.append(TD(counters.get("log4xx")),"\n")
    row.append(TD(counters.get("search")),"\n")
    row.append(TD(counters.get("google")),"\n")
    row.append(TD(IPhist.domains),"\n")
    row.append(TD(counters.get("ROW")),"\n")
    for aType in analyzerData.types:
        row.append(TD(counters.get(aType+".ROW")),"\n")
    return row

def generateHeader(table,types):
    row = TR(TH(),TH("all logentries",colspan="4",bgcolor="#FEBAFE"))
    row.append(TH("collapsed entries",colspan="2",bgcolor="#FEFEBA"))
    row.append(TH("bot ao",BR(),"search",bgcolor="#FEBABA"))
    row.append(TH())
    row.append(TH("filtered entries ROW",colspan="%s"%(len(types)+4),bgcolor="#BAFEFE"))
    row.append(TH("filtered entries Philips",colspan="%s"%(len(types)+1),
                  bgcolor="#BAFEBA"))
    table.append(row)
    colgroup = RawText("<COLGROUP ALIGN=LEFT></COLGROUP>"+
                       "<COLGROUP><COL ALIGN=RIGHT SPAN=%s></COLGROUP>"%
                       (2*len(types)+13))
    table.append(colgroup)
    row = TR(TH("period"),TH("Mbytes"),TH("accesses"))
    row.append(TH("304"),TH("4xx"))
    row.append(TH("collapsed",BR(),"entries"),TH("Unique",BR(),"4xx"),TH("entries"))
    row.append(TH("Google",BR(),"SiteSearch"))
    row.append(TH("#ip"),TH("#top",BR(),"domains"),TH("average",BR(),"hits/IP"))
    for set in ["philips","nonPhilips"]:
        row.append(TH("valid",BR(),"hits"))
        for aType in types:
            if aType=="":
                row.append(TH("without",BR(),"extension"))
            else:
                row.append(TH(aType))
    table.append(row)
def wwwGenerateHeader(table,types):
    row = TR(TH(),TH("all logentries",colspan="4",bgcolor="#FEBAFE"))
    row.append(TH("collapsed entries",colspan="2",bgcolor="#FEFEBA"))
    row.append(TH("search",BR(),"engine",bgcolor="FEBABA"))
    row.append(TH())
    row.append(TH("filtered entries",colspan="%s"%(len(types)+2),bgcolor="#BAFEFE"))
    table.append(row)
    colgroup = RawText("<COLGROUP ALIGN=LEFT></COLGROUP>"+
                       "<COLGROUP><COL ALIGN=RIGHT SPAN=%s></COLGROUP>"%
                       (len(types)+10))
    table.append(colgroup)
    row = TR(TH("period"),TH("Mbytes"),TH("accesses"),TH("304"))
    row.append(TH("errors"),TH("collapsed",BR(),"entries"))
    row.append(TH("errors"),TH("entries"),TH("Google",BR(),"SiteSearch"))
    row.append(TH("domains"),TH("valid hits"))
    for aType in types:
        if aType=="":
            row.append(TH("without",BR(),"extension"))
        else:
            row.append(TH(aType))
    table.append(row)
