from mx.Tools import sizeof
import string

def alldigits(ip):
    ipelements = ip.split(".")
    alldigit = 1
    for el in ipelements:
        alldigit = alldigit and el.isdigit()
    return alldigit

def translateIP(ip,ipDB):    
    if alldigits(ip):
        return ipDB.get(ip)
    else:
        return ip

class DoubleHist:
    def __init__(self,minHits=1):
        self.hist = {}
        self.minHits = minHits
    def add(self,bin,counts):
        if counts>=self.minHits:
            self.hist.setdefault(bin,[0,0])
            self.hist[bin][0]+=1
            self.hist[bin][1]+=counts
    def getHist(self, desc=0):
        result = [(val,domain) for domain,val in self.hist.items()]
        result.sort()
        if desc:
            result.reverse()
        return result
    def getNrEntries(self):
        return len(self.hist)
    def sizeof(self):
        return sizeof(self)+sizeof(self.hist)
    def merge(self,hist2):
        for bin2,val2 in hist2.hist.items():
            if bin2 in self.hist:
                self.hist[bin2][0] += val2[0]
                self.hist[bin2][1] += val2[1]
            else:
                self.hist[bin2] = val2

class Hist:
    def __init__(self):
        self.hist = {}
        self.total = 0
    def add(self,bin,count=1):
        self.hist.setdefault(bin,0)
        self.hist[bin]+=count
        self.total += count
    def getHist(self, desc=0):
        result = [(val,domain) for domain,val in self.hist.items()]
        result.sort()
        if desc:
            result.reverse()
        return result
    def getNrEntries(self):
        return len(self.hist)
    def sizeof(self):
        return sizeof(self)+sizeof(self.hist)
    def merge(self,hist2):
        for bin2,val2 in hist2.hist.items():
            if bin2 in self.hist:
                self.hist[bin2] += val2
            else:
                self.hist[bin2] = val2
            self.total += val2

class Counters:
    def __init__(self, rawEntries, analyzerData ):
        self.dict = {}
        for counter in analyzerData.counters:
            self.dict[counter]=0
        self.dict["rawEntries"] = rawEntries
    def merge(self,counters):
        for counter in counters.dict.keys():
            self.add(counter,counters.dict[counter])
    def add(self,key,val):
        if key in self.dict:
            self.dict[key]+=val
        else:
            self.dict[key]=val
    def get(self,key):
        return self.dict[key]

class IPhist:
    def __init__(self,entries,context,ipDB):
        self.hist=Hist()
        for e in entries:
            self.hist.add(translateIP(e.domain,ipDB))
        self.analyseDomains()
        self.analyseFrequency()
        
    def makeTable(self,analyzerData):
        tab = TableLite(TR(TH("count"),TH("ip address")),"\n")
        sortedIP = self.hist.getHist(1)
        for count,ip in sortedIP:
            tab.append(TR(TD(count),TD(ip), TD(analyzerData.ipDB.getDescription(ip))),"\n")
        return tab

    def merge(self, IPhist2):
        self.hist.merge(IPhist2.hist)
        self.analyseDomains()
        self.analyseFrequency()
        
    def analyseDomains(self):
        self.top= DoubleHist(2)
        for ipAdres in self.hist.hist.keys():
            fields = string.split(ipAdres,".")
            if not alldigits(ipAdres):
                self.top.add(fields[-1],self.hist.hist[ipAdres])
            else:
                self.top.add("alldigital",self.hist.hist[ipAdres])
        self.domains = self.top.getNrEntries()
                
    def analyseFrequency(self):
        self.totalIP=0
        self.freqHist = []
        self.IPaverage = 0.0
        if len(self.hist.hist)>0:
            self.freqHist = (max(self.hist.hist.values())+1)*[0]
            for count in self.hist.hist.values():
                self.freqHist[count]+=1
            totalHits = 0
            for hits in range(2,min(50,len(self.freqHist))):
                self.totalIP += self.freqHist[hits]
                totalHits += self.freqHist[hits]*hits
            if self.totalIP>0:
                self.IPaverage = float(totalHits)/float(self.totalIP)

def main():
    dh = DoubleHist(1)
    dh.add("a",3)
    dh.add("a",2)
    dh.add("b",1)
    dh.add("c",4)
    print dh.getHist()
    print dh.sizeof()
    h = Hist()
    h.add("a")
    h.add("a")
    h.add("b")
    print h.getHist()
    print h.sizeof()
    h2 = Hist()
    h2.add("a")
    h2.add("c")
    h.merge(h2)
    print h.getHist()
    print h.sizeof()
    print "alldigits naam",alldigits("naam")
    print "alldigits 1.2.3.4",alldigits("1.2.3.4")
    
    
if __name__ == '__main__':
    main()
