import string
import os
import os.path
import glob
import stat
from win32com.client.dynamic import Dispatch
import pythoncom
import shutil
import time
import pprint
import zipfile
from GLAcontext import Context
from GLAcontext import LogFile

def compareFilenameTuples(ff1,ff2):
    f1 = string.lower(ff1[0]+ff1[1])
    f2 = string.lower(ff2[0]+ff2[1])
    if f1<f2:
        return -1
    if f1>f2:
        return 1
    return 0

def appendList( destination, list):
    for el in list:
        destination.append(el)
        
def splitList( list, sep=0):
    result = []
    for el in list:
        if sep==0:
            appendList(result, string.split(el))
        else:
            appendList(result, string.split(el,sep))
    return result
        
def decompose( lines ):
    elements = splitList(lines)
    elements = splitList(elements,"{")
    elements = splitList(elements,"(")
    return elements
    
        
class Figure:
    def __init__(self, fileName, fileType, context):
        self.fileName = fileName
        self.fileType = fileType
        self.context = context
        self.pdfFile = context.checkFile(
            os.path.join(self.context.figures,self.fileName+".pdf"))
        if self.pdfFile =="":
            self.pdfMtime = 0
        else:
            self.pdfMtime = os.stat(self.pdfFile)[stat.ST_MTIME]
        if not self.fileName in excludeFigureList:    
            self.vsdFile = context.checkFile(
                os.path.join(self.context.figureSources,self.fileName+".vsd"))
            if self.vsdFile=="":
                self.vsdFile = context.checkFile(os.path.join(self.context.noprefix,self.fileName+".vsd"))
            if self.vsdFile=="":
                self.vsdFile = context.checkFile(os.path.join(self.context.unpublished,self.fileName+".vsd"))
        else:
            self.vsdFile = ""
        if self.vsdFile=="":
            self.vsdMtime = 0
        else:
            self.vsdMtime = os.stat(self.vsdFile)[stat.ST_MTIME]
        self.epsFile = os.path.join(self.context.figures,self.fileName+".eps")
        
    def vsdtoeps(self):
        tempfilename="a%d"%self.context.sequence
        tempfile=os.path.join(self.context.temp,tempfilename+".vsd")
        if os.path.isfile(tempfile):
            os.remove(tempfile)
        shutil.copy(self.vsdFile,tempfile)
        tryCount = 0
        while tryCount<5:
            tryCount += 1
            try:
                doc = self.context.visio.Documents.OpenEx(tempfile,8)
                doc.PrintLandscape = 0
                doc.PrintLandscape = -1
                page = doc.Pages(1)
                tempepsfile = os.path.join(self.context.figures,tempfilename+".eps")
                if os.path.isfile(tempepsfile):
                    self.context.log.log("unexpected epsfile deleted "+tempepsfile)
                    os.remove(tempepsfile)
                page.Export(tempepsfile)
                print self.fileName, " exported"
                doc.Saved = 1
                doc.Close()
                page = 0
                doc = 0
                tryCount = 5
            except pythoncom.com_error,(hr,msg,exc,arg):
                self.context.log.log("***Visio failed with code %d: %s"%(hr,msg))
                if exc is None:
                    self.context.log.log( "There is no extended information")
                else:
                    wcode, source, text, helpFile, helpId, scode = exc
                    self.context.log.log( "the source of the error is: "+ source)
                    self.context.log.log( "the error message is: "+ text)
                    self.context.log.log( "more info can be found in %s (id=%d)"%(helpFile,helpId))
                    self.context.log.info("com interfacecount: %d"%pythoncom._GetInterfaceCount())
                if tryCount>4:
                    raise RuntimeError,"Visio failed too many times"
                print "before Quit ", self.context.visio
                self.context.visio.Quit()
                time.sleep(1)
                self.context.visio = Dispatch("Visio.Application")
                print "after restart: ", self.context.visio
        os.remove(tempfile)
        if os.path.isfile(self.epsFile):
            os.remove(self.epsFile)
        os.rename(tempepsfile, self.epsFile)

    def epstopdf(self):
        os.chdir(self.context.figures)
        fd = os.popen("epstopdf "+self.fileName+".eps")
        result = fd.read()
        self.context.log.log(result)
        fd.close()
        self.pdfFile = os.path.join(self.context.figures,self.fileName+".pdf")
        if not os.path.isfile(self.pdfFile):
            self.pdfFile = ""
        else:
            self.pdfMtime = os.stat(self.pdfFile)[stat.ST_MTIME]
            
    def generate(self): 
        self.context.log.info( "generate "+ self.fileName)
        self.vsdtoeps()
        self.epstopdf()
        os.remove(self.epsFile)

class Document:
    def __init__(self,rootName,mainFile,context,type):
        self.rootName = rootName
        self.context = context
        self.type = type
        self.sourceDir = os.path.join(context.documentSources,rootName)
        if not os.path.isdir(self.sourceDir):
            self.context.log.log("docSourceDir "+self.sourceDir+" does not exist")
            return
        self.mainFile = mainFile
        self.validFile = os.path.isfile(os.path.join(self.sourceDir,self.mainFile+".tex"))
        if not self.validFile:
            self.context.log.log(self.mainFile+".tex does not exist")
            return
        self.texTempLogFile = os.path.join(self.context.temp,self.mainFile+".log")
        self.texLogFile = os.path.join(self.context.texLog,self.mainFile+".log")
        self.tempPdfFile = os.path.join(self.context.temp,self.mainFile+".pdf")
        self.pdfFile = os.path.join(self.context.documents,self.mainFile+".pdf")
        if not os.path.isfile(self.texLogFile):
            self.context.log.log(self.texLogFile+" does not exist")
            self.pdfLatex()
            self.pdfMtime = 0
        else:
            if os.path.isfile(self.pdfFile):
                self.pdfMtime = os.stat(self.pdfFile)[stat.ST_MTIME]
            else:
                self.pdfMtime=0
        self.figures = {}
        self.texFiles = []
            
    def pdfLatex(self):
        os.chdir(self.sourceDir)
        commandline = "pdflatex -quiet -max-print-line=200 "
        commandline +="-include-directory="+ self.context.temp
        commandline +=" -output-directory="+self.context.temp+" "+ self.mainFile
        self.context.log.log(commandline)
        fd = os.popen(commandline)
        result = fd.read()
        self.context.log.log(result)
        fd.close()
        if string.find(result,"Emergency stop")>0:
            self.context.log.log("pdflatex emergency stop")
            print "======pdflatex terminal output:============"
            print result
            raise RuntimeError,"pdflatex emergency stop"
        try:
            if os.path.isfile(self.pdfFile):
                os.remove(self.pdfFile)
            os.rename(self.tempPdfFile,self.pdfFile)
            self.pdfMtime = os.stat(self.pdfFile)[stat.ST_MTIME]
        except os.error:
            self.context.log.log("could not remove or rename "+
                                 self.tempPdfFile +" to "+self.pdfFile+
                                 " file in use in acrobat?")
            self.pdfFile = ""
        if os.path.isfile(self.texLogFile):
            os.remove(self.texLogFile)
        shutil.copy(self.texTempLogFile,self.texLogFile)

    def extractWarnings(self):
        if not os.path.isfile(self.texLogFile):
            self.context.log.log( "logfile "+self.texLogFile+" does not exist")
            return
        fd = open(self.texLogFile)
        latexOutput = fd.readlines()
        for i in range(len(latexOutput)):
            message = string.lower(latexOutput[i])
            if string.find(message,"warning")!=-1 or string.find(message,"error")!=-1:
                self.context.log.log( "latex : "+latexOutput[i-2]+latexOutput[i-1]+latexOutput[i])
        fd.close()
        
    def bibtex(self):
        os.chdir(self.context.temp)
        commandline="bibtex "+ self.mainFile
        self.context.log.log(commandline)
        fd = os.popen(commandline)
        log = fd.read()
        fd.close()
        return log
    
    def generatePaper( self):
        self.context.log.log( "---------------generate "+self.mainFile)
        self.generateRenewals()
        self.pdfLatex()
        if self.generateRenewals()>0:
            self.pdfLatex()
        bibtexOutput = self.bibtex()
        if string.find(bibtexOutput,"rror")!=-1 or string.find(bibtexOutput,"arning")!=-1:
            self.context.log.log( "bibtex: \n"+bibtexOutput)
        self.pdfLatex()
        self.pdfLatex()
        self.extractWarnings()
        
    def generateSlides( self):
        self.context.log.log( "---------------generate "+self.mainFile)
        self.generateRenewals()
        self.pdfLatex()
        if self.generateRenewals()>0:
            self.pdfLatex()

    def generate( self):
        if self.type == "slides":
            self.generateSlides()
        elif self.type == "paper":
            self.generatePaper()
        else:
            self.context.log.log("unknown document type: "+self.type)

    def getTexFiles(self):
        # determines which texfiles are used in this document,
        # by looking for all .tex files in the logfile
        try:
            fd = open(self.texLogFile)
        except:
            self.log.log( "latex log file: "+self.texLogFile+"could not be opened")
            return []
        parts = decompose(fd.readlines())
        for part in parts:
            texIndex = string.find(part,".tex")
            if texIndex>0:
                fileName = part[0:texIndex+4]
                self.texFiles.append(fileName)
        fd.close()
        return self.texFiles
      
    def toBeGenerated(self):
        # determines document needs to be generated
        # 0 means generation not needed or impossible
        # <>0 generation required
        if not self.validFile:
            self.context.log.log(self.mainFile+ " invalid main file")
            return 0
        os.chdir( self.sourceDir )
        if self.pdfMtime == 0:
            return "no pdf file"
        self.getTexFiles()
        self.context.log.info( "texFiles:"+str(self.texFiles))
        for name in self.texFiles:
            if len(os.path.dirname(name))>0:
                fullName = name
            else:
                fullName = os.path.join(self.sourceDir,name)
            if not os.path.isfile(fullName):
                self.context.log.log("Panic: "+fullName+" does not exist?")
                return 0
            texMtime = os.stat(fullName)[stat.ST_MTIME]
            if texMtime>self.pdfMtime:
                return "tex file newer than pdf file"
        self.getFigures()
        self.context.log.info("figures: "+ str(self.figures.keys()))
        self.determineRenewals()
        self.context.log.info("renewals: "+ str(self.renewals))
        if len(self.renewals)>0:
            return "new figures generated"
        for figure in self.figures.values():
            if figure.pdfMtime>self.pdfMtime:
                return figure.fileName + " newer than pdf document"
        return 0

    def getFigures(self):
        try:
            fd = open(self.texLogFile)
        except:
            self.context.log.log( "latex log file: "+self.texLogFile+"could not be opened")
            return {}
        for line in fd.readlines():
            figuresIndex = string.find(line,"/gaudi/cache/figures")
            pdfIndex = string.find(line,".pdf")
            ErrorIndex = string.find(line,"Error")
            if figuresIndex>=0:
                if ErrorIndex>=0:
                    restLine = line[figuresIndex+len("/gaudi/cache/figures/"):]
                    endIndex=string.find(restLine,"'")
                    fileName = restLine[0:endIndex]
                    if not self.figures.has_key(fileName):
                        self.figures[fileName]=Figure(fileName,"unknown",self.context)
                if pdfIndex>=0:
                    fileName = line[figuresIndex+len("/gaudi/cache/figures/"):pdfIndex]
                    if not self.figures.has_key(fileName):
                        self.figures[fileName]=Figure(fileName,".pdf",self.context)
        fd.close()
        return
        
    def determineRenewals(self):
        self.renewals = []
        for figure in self.figures.values():
            if os.path.isfile(figure.vsdFile):
                if os.path.isfile(figure.pdfFile):
                    if figure.vsdMtime>figure.pdfMtime:
                        os.remove(figure.pdfFile)
                        self.renewals.append(figure.fileName)
                else:
                    if string.find(figure.fileType,".pdf")==0:
                        self.context.log.info("figure "+figure.fileName+".pdf, does not exist anymore.")
                    self.renewals.append(figure.fileName)
            else:
                if not figure.fileName in excludeFigureList:
                    self.context.log.log("file "+figure.fileName+".vsd does not exist")

    def generateRenewals(self):
        self.getFigures()
        self.determineRenewals()
        try:
            if len(self.renewals)>0:
                if self.context.visio==0:
                    self.context.visio = Dispatch("Visio.Application")
                    print "self.context.visio: ", self.context.visio
                for file in self.renewals:
                    self.context.log.info( "generate: "+file)
                    self.figures[file].generate()
                    self.context.sequence+=1
            return len(self.renewals)
        except pythoncom.com_error,(hr,msg,exc,arg):
            self.context.log.log("application failed with code %d: %s"%(hr,msg))
            if exc is None:
                self.context.log.log( "There is no extended information")
            else:
                wcode, source, text, helpFile, helpId, scode = exc
                self.context.log.log( "the source of the error is: "+ source)
                self.context.log.log( "the error message is: "+ text)
                self.context.log.log( "more info can be found in %s (id=%d)"%(helpFile,helpId))
                self.context.log.info("com interfacecount: %d"%pythoncom._GetInterfaceCount())
            raise RuntimeError,"Visio failed"

    def close(self):
        if self.context.visio<>0:
            self.context.visio.Quit()
            self.context.visio=0

postfixDefs = { "Slides":"slides",
                "Paper":"paper",
                "Book":"paper",
                "allSlides":"slides"}
excludeFigureList = ["EmbleemPlusName","logoTBD",]
class RootDocument:
    def __init__(self, context, root, postfix):
        self.context = context
        self.root = root
        self.postfix = postfix
        self.docs = []
        self.sourceDir = os.path.join(context.documentSources,root)
        self.zip = os.path.join(self.context.zip,self.root+".zip")
        self.textzip = os.path.join(self.context.local,root+"Text.zip")
        self.changed = 0
        if not os.path.isdir(self.sourceDir):
            self.context.log.log("docSourceDir "+self.sourceDir+" does not exist")
            return
        for postfix in self.postfix:
            if os.path.isfile(os.path.join(self.sourceDir,self.root+postfix+".tex")):
                self.docs.append(Document(root,root+postfix,context,postfixDefs[postfix]))
                self.context.log.info("existing mainfile: "+self.root+postfix+".tex")

    def generateDocument(self):
        for doc in self.docs:
            toGenerate = doc.toBeGenerated()
            figureFile = open(os.path.join(self.context.figCrossRef,doc.mainFile+".txt"),
                              "w")
            for figure in doc.figures.keys():
                figureFile.write(figure+"\n")
            figureFile.close()
            if toGenerate<>0:
                self.context.log.log(toGenerate)
                doc.generate()
                self.changed = 1
                
    def determineZipToBeGenerated(self):
        return self.changed or not os.path.isfile(self.zip)
    
    def determineFigures(self):
        self.allFigures = {}
        for doc in self.docs:
            self.allFigures.update(doc.figures)
        for exclude in excludeFigureList:
            if exclude in self.allFigures:
                del self.allFigures[exclude]

    def generateZip( self ):
        allzipfile = zipfile.ZipFile(self.zip,"w",zipfile.ZIP_DEFLATED)
        textzipfile = zipfile.ZipFile(self.textzip,"w",zipfile.ZIP_DEFLATED)
        figurePath = os.path.splitdrive(self.context.figureSources)[1]
        sourcePath = os.path.splitdrive(self.sourceDir)[1]
        for figure in self.allFigures:
            file = os.path.join(figurePath,figure+".vsd")
            file = self.context.checkFile(file," --> should be in ExcludeList ")
            if len(file)>2:
                allzipfile.write(file)
        os.chdir("/")
        for texfile in glob.glob(os.path.join(sourcePath,"*.tex")):
            allzipfile.write(texfile)
            textzipfile.write(texfile)
        for txtfile in glob.glob(os.path.join(sourcePath,"*DocDef.txt")):
            allzipfile.write(txtfile)
            textzipfile.write(txtfile)
        allzipfile.close()
        textzipfile.close()
            
def generateDocuments( context, roots, postfix ):
    for root in roots:
        context.log.log("document: "+ root)
        rootDoc = RootDocument(context, root,postfix)
        rootDoc.generateDocument()
        if rootDoc.determineZipToBeGenerated():
            rootDoc.determineFigures()
            rootDoc.generateZip()

def main():               
    drive = os.path.splitdrive(os.getcwd())[0]
    rootdir = os.path.join(drive,"\\gaudi")
    logFile = LogFile(os.path.join(rootdir,"temp/generateDocuments"),toWindow=1,showInfo=0)
    context = Context(rootdir,logFile)
    startTime = time.time()
    logFile.log( "started at: "+ time.ctime(startTime))
    books = [
##        "ArchitecturalReasoning",
##        "ComposableArchitectures",
##        "CourseDescriptions",
##        "HumanMeasure",
##        "OOTIrequirements",
        "SupportingProcesses",
##        "SystemArchitecture",
##        "Thesis",
        ]
    courses = [
##        "CAFCR",
##        "EA",
        "ESA",
##        "EScontext",
##        "OOTI",
        "SARCH",
        "ShortSARCH",
        "Performance",
        ]
    modules = [
##        "ModuleMICAFCR",
##        "ModuleArchitectingMethodOverview",
##        "ModuleCAFCRinfo",
##        "ModuleCAFCRwrapUp",
##        "ModuleCAviews",
##        "ModuleCRviews",
##        "ModuleEAinfo",
##        "ModuleEAconcepts",
##        "ModuleESAinfo",
##        "ModuleESCinfo",
##        "ModuleFunctionalView",
##        "ModuleHRM",
##        "ModuleHumanSide",
##        "ModuleManagementPresentation",
##        "ModuleProductFamilies",
##        "ModuleQualities",
##        "ModuleRequirements",
##        "ModuleRoadmapping",
##        "ModuleRoleAndTask",
##        "ModuleRoleOfSoftware",
##        "ModuleSARCHinfo",
##        "ModuleSARCHwrapUp",
##        "ModuleSSARCHinfo",
##        "ModuleStoryTelling",
##        "ModuleSystemArchitectToolkit",
##        "ModuleSystemArchitectureContext",
##        "ModuleSupportingProcesses",
        ]
    documents = [
##        "AggregationLevels",
##        "AgileOrganisation",
##        "ApplicationView",
##        "ArchitectingAndStandardization",
##        "ArchitectingUserValue",
        "ArchitecturalDecisionMaking",
##        "ArchitecturalRefactoring",
##        "ArchitecturalRefactoringMR",
##        "ArchitecturalThinking",
##        "ArchitectAndHumanMeasure",
##        "ArchitectingMethodOverview",
##        "ArchitectingResearchMethod",
##        "ArchitectingStyles",
##        "ArchitectManagementInteraction",
##        "ARevaluation",
##        "ARevaluationOther",
##        "BasicCAFCR",
##        "BasicWorkingMethodArchitect",
##        "BloatingExplored",
        "BodercLessonsLearned",
##        "CAFCRcourse",
##        "CommunicatingViaCAFCR",
##        "ConceptualView",
##        "CriterionsForArchitectingMethods",
##        "CustomerObjectivesView",
        "DecomposingTheArchitect",
##        "DidacticRecommendationsSE",
##        "DocumentationGranularity",
##        "DYOF",
##        "EAcourse",
##        "EAsoftRealTime",
##        "EmbeddedSystemsOpportunities",
##        "ESAcourse",
##        "ESAfuture",
##        "ESCcourse",
##        "ESIpositioning",
##        "ESIcapabilities",
##        "ESresearchAgenda",
##        "ExecutionArchitectureApproach",
##        "ExecutionArchitectureConcepts",
##        "ExperienceTransfer",
##        "ExploringByMeasuringInstrumenting",
##        "Feedback",
##        "FromAutonomousToIntegrated",
##        "FromFuzzyToSmart",
##        "FromIndustryToKnowHow",
##        "FunctionalView",
##        "FunctionProfiles",
##        "FutureOfArchitectingResearch",
##        "GapIndustryAcademics",
##        "GaudiDocumentationToolsReq",
##        "GaudiProject",
##        "GaudiWebSiteReq",
##        "GenericDevelopments",
##        "GenericityVsSpecificity",
##        "HowToAppraise",
##        "HowtoManageablePlatformArchitecture",
##        "HumanMeasure",
##        "HumanSide",
##        "ImportanceOfSAforDevelopment",
##        "IndustryAsLaboratoryApplied",
##        "InformalNatureSystemsEngineering",
##        "IntegratingCAFCR",
##        "IntroductionMedicalImagingCase",
##        "IRCwaferstepper",
##        "LekenPraatje",
##        "LightWeightArchitecting",
##        "LightWeightArchitectingRevisited",
##        "LightWeightReview",
##        "Maturity",
##        "MarketProductLifecycle",
##        "MeddlerOrSavior",
##        "MedicalImaging",
##        "MIchronological",
##        "MIconceptualView",
##        "MIstories",
##        "MIstoryToDesign",
##        "MIthreadsOfReasoning",
##        "MIviewsCAF",
        "ModelingHierarchyDynamicRange",
##        "MultiDomainMethods",
##        "MVillustration",
##        "OOTIcourse",
        "OverviewHowTo",
        "PerformanceEngineering",
        "PerformanceHRT",
        "PerformanceIntro",
        "PerformanceMethodFundamentals",
        "PerformancePatterns",
        "PerformanceSRT",
##        "PFanalysisAndDefinition",
##        "PositioningArchitecturalReasoning",
##        "PositioningCAFCRinBusiness",
##        "PositionOfResearch",
##        "PresentationMITOR",
##        "ProcessDecompositionOfBusiness",
##        "ProductCreationProcess",
##        "QualityNeedles",
        "QuantifiedSystemDesign",
##        "RealizationView",
##        "ReflectionOnResearchMethod",
##        "RelationArchitectManager",
##        "Requirements",
##        "Roadmapping",
##        "RoadmappingInStrategyProcess",
##        "RoleArchitectTurbulentWorld",
##        "RoleInInnovation",
##        "RoleOfSoftware",
##        "RoleSystemArchitect",
##        "SARCHcourse",
##        "SARCHexperience",
##        "SemiconductorSoftwareStrategy",
##        "ShortSARCHcourse",
##        "SimplisticFinancialComputations",
##        "SoftwareProductivity",
##        "SoftwareReuse",
##        "StoryHowTo",
##        "SystemArchitectureProcess",
        "SystemIntegrationHowTo",
##        "SystemsAndAgility",
##        "TCAFsubMethods",
##        "TCRsubMethods",
##        "TemplateHowTo",
##        "TheSilverBullet",
##        "ThesisBWMA",
##        "ThesisPropositions",
##        "ThesisStellingen",
##        "ThesisStory",
##        "ThreadsOfReasoning",
##        "TutorialSoftwareAsIntegratingTechnology",
##        "ValidationOfRequirements",
##        "ValueChainAndDrivers",
##        "WDCarchitecting",
##        "WhatIsAProcess",
##        "WhatIsArchitecting",
        "WorkshopHowTo",
        ]
    generateDocuments(context, documents,["Paper","Slides"])
    generateDocuments(context, books,["Book"])
    generateDocuments(context, modules,["Paper","Slides"])
    generateDocuments(context, courses,["allSlides"])
    endTime = time.time()
    logFile.log("finished at: "+ time.ctime(endTime))
    logFile.log("total elapsed time:%7.1f seconds"%(endTime - startTime))
    logFile.close()

if __name__ == '__main__':
    main()
    
