from HTMLgen import *
import string
import os.path
import time
from GLAcontext import Context
from GLAcontext import LogFile

def include( fileName ):
    try:
        return open("./"+fileName).read()
    except IOError, e:
        print "Include could not open ", fileName
        
def inScope( scope1, scope2 ):
# this works only for nlww and www :-)
    return scope1 >= scope2

def sortOnDate(i,j):
    return cmp(j["id"],i["id"])

def getPageTitle( pageDefinition ):
    if pageDefinition.has_key("navigationTitle"):
        title = pageDefinition["navigationTitle"]
    else:
        title = pageDefinition["Title"]
    return title

def setDefaultScope( definition ):
    if not definition.has_key("scope"):
        definition["scope"]="www"
        
def getDefinitionsInSubDirs( path,nameTag ):
    dirs = filter(lambda d,p=path: os.path.isdir(os.path.join(p,d)), os.listdir(path))
    definitions = {}
    for dir in dirs:
        defFiles = filter(lambda i,n=nameTag:string.find(i,n)>-1, os.listdir(path+"/"+dir))
        for file in defFiles:
            att = readAttributes(path+"/"+dir+"/",file)
            att["id"] = file[0:string.find(file,nameTag)]
            definitions[att["id"]]=att
    return definitions

def getDefinitionsInDir( path,nameTag ):
    definitions = {}
    defFiles = filter(lambda i,n=nameTag:string.find(i,n)>-1, os.listdir(path))
    for file in defFiles:
        att = readAttributes(path+"/",file)
        att["id"] = file[0:string.find(file,nameTag)]
        definitions[att["id"]]=att
    return definitions

def readAttributes( path, file ):
    dict = {}
    inFile = open( path+file, "r")
    input = inFile.readlines()
    inFile.close()
    lines = []
    linenr = 0
    while linenr<len(input):
        lines.append(input[linenr].strip())
        while input[linenr].endswith(",\n"):
            linenr += 1
            lines[-1]+=input[linenr].strip()
        linenr += 1
    for line in lines:
        if line<>"":
            attributes = string.split(line,"=",1)
            key = string.strip(attributes[0])
            try:
                value = string.strip(attributes[1])
            except:
                print path,"/",file
                print attributes, "= missing?"
            dict[key] = value
            if key in ["Documents","Column1","newDoc","newPage","updateDoc","updatePage"]:
                dict[key]=[e.strip() for e in value.split(",")]
    return dict
     
def generateIcons( content):
    icons = TD(Class="icons")
    icons.append(Image("square"+content["status"]+".gif"),"\n")
    contentType = content["type"]
    if contentType in ["Article","Paper"]:
        icons.append(Href(content["id"]+"Paper.pdf", Image("paper.gif",border=0)),"\n")
    if contentType in ["Article","Presentation"]:
        icons.append(Href(content["id"]+"Slides.pdf", Image("slide.gif",border=0)),"\n")
    if contentType=="Book":
        icons.append(Href(content["id"]+"Book.pdf", Image("book.gif",border=0)),"\n")
    if contentType=="SlideBook":
        icons.append(Href(content["id"]+"allSlides.pdf", Image("SlideBook.gif",border=0)),"\n")
    if contentType in ["Article","Paper","Presentation","Book","SlideBook"]:
        if os.path.isfile(os.path.join(rootdir,"www/sources",content["id"]+".zip")):
            icons.append(Href("sources/"+content["id"]+".zip", Image("src.gif",border=0)),"\n")
    return icons

def generateElement( content):
    el = TR()
    if content["type"]=="Link":
        el.append(TD(RawText(content["text"]),**{"class":"link"}), TD())
    else:
        if content["type"]=="href":
            el.append(TD(Href(content["id"]+".pdf",RawText(content["title"])),**{"class":"href"}))
            return el
        title = TD(RawText(content["title"]),**{"class":"title"})
        el.append(title,"\n")
        el.append(generateIcons(content),"\n")
    if "occasion" in content:
        annotation = TD(**{"class":"occasion"})
        annotation.append(RawText(content["occasion"]),"; ",RawText(content["occasionDate"]))
        if "location" in content:
            annotation.append(" in ",RawText(content["location"]))
        el.append(annotation)
    return el

def generateDocChangeRow( definitions, date, page, content, changeType, showIcons, showDate):
    el = TR()
    if showDate:
        el.append(TD(date),"\n")
    else: el.append(TD(),"\n")
    el.append(TD(changeType),"\n")
    if content["type"]=="href":
        el.append(TD(Href(content["id"]+".pdf",RawText(content["title"])),width=300),TD(),"\n")
    else:
        if content["type"]=="Link":
            el.append(TD(RawText(content["text"]),width=300),TD(),"\n")
        else:
            title = TD(RawText(content["title"]), width =300,**{"class":"title"})
            el.append(title,"\n")
            if showIcons:
                el.append(generateIcons(content),"\n")
            else:
                el.append(TD(),"\n")
    if page<>"":
        pageref = TD(" at page  ")
        pageref.append(Href(definitions["page"][page]["id"]+".html",definitions["page"][page]["pageTitle"]))
        el.append(pageref)
    else:
        el.append(TD())
    return el

def generatePageChangeRow( definitions, docId, date, changeType, scope, table, showDate ):
    pageDef = definitions["page"]
    if not pageDef.has_key(docId):
        print "Change ", date, "doc: ",docId, " not found"
        return showDate
    page = pageDef[docId]
    if not inScope(page["scope"], scope):
        print "Change ", date, "doc: ", docId, "not in scope ",scope
        return showDate
    el = TR()
    if showDate:
        el.append(TD(date),"\n")
    else: el.append(TD(),"\n")
    el.append(TD(changeType),"\n")
    el.append(TD(Href(page["id"]+".html",page["pageTitle"]),
                 width=300,**{"class":"title"}))
    table.append(el)
    return 0
    
def generateTextChangeRow( date, text, showDate):
    el = TR()
    if showDate:
        el.append(TD(date),"\n")
    else: el.append(TD(),"\n")
    el.append(TD())
    el.append(TD(text,width=300,**{"class":"title"}),"\n")
    return el

def generateColumn( definitions, volumes, scope ):
    table = TableLite( cellpadding=0 )
    for volumeName in volumes:
        if definitions["volume"].has_key(volumeName):
            if inScope(definitions["volume"][volumeName]["scope"],scope):
                table.append(TR(TD(definitions["volume"][volumeName]["Title"],**{"class":"volume"})),"\n")
                if "Documents" in definitions["volume"][volumeName]:
                    for content in definitions["volume"][volumeName]["Documents"]:
                        if content in definitions["doc"]:
                            defContent = definitions["doc"][content]
                            if inScope(defContent["scope"], scope ):
                                table.append(generateElement(defContent))
                        else:
                            print "unknown document: ", content, " in volume: ",volumeName
                else:
                    print "no documents in volume: ", volumeName
        else:
            print "no definition for section: ", volumeName
    return TD(table, width="65%",valign="TOP")

def findVolumeOfDoc( definitions, docId):
    for volume in definitions["volume"].keys():
        if docId in definitions["volume"][volume].get("Documents",[]):
            return volume
    return ""
    
def findPageOfVolume( definitions, volume):
    for page in definitions["page"].keys():
        if volume in definitions["page"][page].get("Column1",[]):
            return page
    return ""

def generateDocChange( definitions, docId, date, changeType,refferedDocs, scope, table, showDate ):
    docDef = definitions["doc"]
    if not docDef.has_key(docId):
        print "Change ", date, "document: ",docId, " not found"
        return showDate
    doc = docDef[docId]
    if not inScope(doc["scope"], scope):
        print "Change ", date, "document: ", docId, " not in scope ",scope
        return showDate
    volume = findVolumeOfDoc( definitions, docId)
    if volume == "":
        print "Change ", date, " no volume found referring to document: ", docId
    page = findPageOfVolume( definitions, volume)
    if page == "":
        print "Change ", date, " no page found referring to volume: ", volume
    freshDoc = not docId in refferedDocs
    row = generateDocChangeRow(definitions,date,page,doc,changeType,freshDoc,showDate)
    if freshDoc:
        refferedDocs.append(docId)
    table.append(row)
    return 0

def generateChangeLog(definitions,scope):
    changes = definitions["change"].values()
    bgcolor=definitions["color"]["titleBackground"]    
    refferedDocs = []
    changes.sort(sortOnDate)
    table = TableLite( cellpadding=0 )
    for change in changes:
        if inScope(change["scope"],scope):
            date = change["id"]
            showDate = 1
            if change.has_key("title"):
                table.append(generateTextChangeRow( date, RawText(change["title"]),
                                                    showDate))
                showDate = 0
            if change.has_key("newDoc"):
                for docId in change["newDoc"]:
                    showDate = generateDocChange(definitions, docId, date, Bold("new"),
                                                 refferedDocs,scope,table,showDate)
            if change.has_key("newPage"):
                for pageId in change["newPage"]:
                    showDate = generatePageChangeRow(definitions,pageId,date,Bold("new"),
                                                 scope, table,showDate)
            if change.has_key("updateDoc"):
                for docId in change["updateDoc"]:
                    showDate = generateDocChange(definitions, docId, date, "update",
                                             refferedDocs,scope,table,showDate)
            if change.has_key("updatePage"):
                for pageId in change["updatePage"]:
                    showDate = generatePageChangeRow(definitions,pageId,date,
                                                 "update",scope, table,showDate)
            if change.has_key("text"):
                table.append(generateTextChangeRow(date, RawText(change["text"]),
                                                   showDate))
    return table

def generatePage( definitions, page, scope, context ):
    if not definitions["page"].has_key(page):
        print "generatePage: ", page, " not defined"
        return
    pageDefinition = definitions["page"][page]
    pageType = pageDefinition["type"]
    page = pageDefinition["id"]
    if pageType in ["Include","IncludeWithLegend","Index","ChangeLog"]:
        doc = SimpleDocument(title=pageDefinition["pageTitle"],bgcolor=definitions["color"]["pageBackground"],
                             stylesheet="gaudi.css")
        if pageDefinition.has_key("Keywords"):
            doc.meta = Meta(name="keywords",content=pageDefinition["Keywords"])
        if pageDefinition.has_key("Title"):
            doc.append(Header(2,RawText(pageDefinition["Title"])))
            doc.append(Div(RawText(include("pageSources/navigation.html")),"\n",**{"class":"navcolumn"}))
        if pageType=="Index":
            doc.append(Div(generateColumn(definitions,pageDefinition["Column1"],scope),**{"class":"contentcolumn"}),"\n")
            doc.append(Div(RawText(include("pageSources/legend.html")),"\n",**{"class":"legendcolumn"}))
        if pageType=="Include":
            doc.append(Div(RawText(include("pageSources/"+pageDefinition["File"])),**{"class":"contentcolumn"}),"\n")
        if pageType=="IncludeWithLegend":
            doc.append(Div(RawText(include("pageSources/"+pageDefinition["File"])),**{"class":"contentcolumn"}), "\n" )
            doc.append(Div(RawText(include("pageSources/legend.html")),**{"class":"legendcolumn"}),"\n")
        if pageType=="ChangeLog":
            doc.append(Div(generateChangeLog(definitions,scope),**{"class":"historycolumn"}),"\n")
        if scope=="www":
            doc.write(os.path.join(context.www,page+".html"))
        else:
            doc.write(os.path.join(context.local,page+".html"))
    else:
        if pageType<>"Obsolete":
            print page, " : ", pageType, "not yet implemented"

def readAllDefinitions():
    definitions={}
    definitions["color"]={ "pageBackground" : "#EFEFEF", "titleBackground" : "#FFFFFF"}
    definitions["page"] = getDefinitionsInDir( "pageSources", ".PageDef." )
    map(setDefaultScope,definitions["page"].values())
    for p in definitions["page"].values():
        p["pageTitle"] = getPageTitle(p)
    definitions["doc"] = getDefinitionsInSubDirs( "documentSources",".DocDef." )
    map(setDefaultScope,definitions["doc"].values())
    definitions["volume"] = getDefinitionsInDir( "pageSources", ".VolDef." )
    map(setDefaultScope,definitions["volume"].values())
    definitions["change"] = getDefinitionsInDir( "changelog", ".Change." )
    map(setDefaultScope,definitions["change"].values())
    return definitions

drive = os.path.splitdrive(os.getcwd())[0]
rootdir = os.path.join(drive,"/gaudi")

def main():
    startTime = time.time()
    print "started at: ", time.ctime(startTime)
    os.chdir(os.path.join(rootdir,"src"))
    logFile = LogFile(os.path.join(rootdir,"temp/generateDocuments"),toWindow=1,showInfo=0)
    context = Context(rootdir,logFile)
    definitions = readAllDefinitions()
    defTime = time.time()
    print "definitions: ", time.ctime(defTime), " definitions elapsed time: ", defTime - startTime
    for scope in ["www"]:
##, "nlww"]:
        for pageDefinition in definitions["page"].values():
            generatePage( definitions, pageDefinition["id"], scope, context )
    endTime = time.time()
    print "finished at: ", time.ctime(endTime), "generation elapsed time: ", endTime - defTime
    print "total elapsed time: ", endTime - startTime, " seconds"

if __name__ == '__main__':
    main()
