import sys

class RangeDict:
    def __init__(self):
        self.ranges = []

    def convertRange( self, range ):
        if range =="*":
            return 0,255
        elif "-" in  range:
            min, max = range.split("-")
            return int(min), int(max)
        else:
            n = int(range)
            return n,n

    def add( self, range, value ):
        min, max = self.convertRange(range)
        self.ranges.append((min,max,value))

    def get( self, key ):
        key = int(key)
        for min,max,val in self.ranges:
            if key>=min and key <= max:
                return val
        return None

    def check( self, range ):
        minIn,maxIn = self.convertRange(range)
        for min,max,val in self.ranges:
            if minIn>=min and maxIn<=max:
                return 1
        return 0

class IPdb:
    def __init__(self,filename):
        defs = open(filename,"r").readlines()
        self.top = {}
        self.domainDict = {}
        lineNr = 1
        for line in defs:
            el = line.split(";")
            if len(el)<4:
                print "insufficient fields in line ",lineNr, line
                break
            iprange = el[0].strip()
            domain = "*"+el[1].strip()
            domainDescr = el[2].strip()
            source = el[3].strip()
            try:
                ip1,ip2,ip3,ip4 = iprange.split(".")
            except ValueError:
                print "wrong length of ip address", iprange
                sys.exit()
            ip3 = ip3.strip()
            ip4 = ip4.strip()
            topIP=ip1+"."+ip2
            if topIP in self.top:
                subDict = self.top[topIP]
            else:
                subDict = RangeDict()
                self.top[topIP]=subDict
            if subDict.check(ip3):
                subsubDict = subDict.get(ip3)
            else:
                subsubDict = RangeDict()
                subDict.add(ip3,subsubDict)
            if subsubDict.check(ip4):
                print "double definition: ", ip1+"."+ip2+"."+ip3+"."+ip4
            else:
                subsubDict.add(ip4,domain)
                self.domainDict[domain]=(domainDescr,source)
            lineNr += 1

    def get(self,ipadres):
            ip1,ip2,ip3,ip4 = ipadres.split(".")
            ip3 = ip3.strip()
            ip4 = ip4.strip()
            topIP=ip1+"."+ip2
            if topIP in self.top:
                subDict = self.top[topIP]
                if subDict.check(ip3):
                    subsubDict = subDict.get(ip3)
                    if subsubDict.check(ip4):
                        return subsubDict.get(ip4)
            return ipadres
        
    def getDescription( self, domain ):
        if domain in self.domainDict:
            return self.domainDict[domain][0]
        return ""
        
    def show(self):
        for el in self.top:
            print "topIP: ",el,
            for el2 in self.top[el].ranges:
                print "el2 raw: ",el2
                print "el2: ",el2[0],el2[1],"[",
                for el3 in el2[2].ranges:
                    print el3,
                print "]"

def main():
    db = IPdb("g:\\gaudi\\src\\PythonTools\\ipreverseNew.txt")
##    db.show()
    print db.get("212.217.66.3")
    print db.get("1.1.1.1")
    print db.get("64.80.70.190")
    print db.get("203.145.186.2")
    print db.getDescription("*wanadoo.co.ma")
    
if __name__ == '__main__':
    main()


            
